package com.db4o.lib;

import com.db4o.*;
import java.lang.reflect.*;
import java.util.*;
import java.io.*;

public class Logger
{
	private static int maximumDepth = Integer.MAX_VALUE;
	private static PrintStream out = System.out;
	private static String cr = "";
	private static String sp = " ";
	
	public static void setOut(PrintStream ps){
		out = ps;
	}
	
	public static void setCarriageReturn(String carriageReturn){
		cr = carriageReturn;
	}
	
	public static void setSpace(String space){
		sp = space;
	}
	
	public static void setMaximumDepth(int depth){
		maximumDepth = depth;
	}
	
	public static boolean silent = false;

	public static void log(ObjectContainer a_container, Object a_object){
		if(a_object == null){
			log("[NULL]");
		}else{
			log(a_object.getClass().getName());
			log(a_container, a_object, 0);
		}
	}

	static void log(ObjectContainer a_container, Object a_object, int a_depth){
		if(a_depth > maximumDepth){
			return;
		}
		
		Class[] classes = getClassHierarchy(a_object);

		String spaces = "";
		for(int i = classes.length - 1; i >= 0; i--){
			spaces = spaces + sp;

			String className = spaces;
			int pos = classes[i].getName().lastIndexOf(".");
			if(pos > 0){
				className += classes[i].getName().substring(pos);
			}else{
				className += classes[i].getName();
			}
			
			if(classes[i] == Date.class){
				String fieldName = className + ".getTime";
				Object obj = new Long(((Date)a_object).getTime());
				log(a_container, obj, Long.class,fieldName, a_depth + 1, -1);
				
			}else{
				Field[] fields = classes[i].getDeclaredFields();
				for (int j = 0; j < fields.length; j++){
					Platform.setAccessible(fields[j]);

					String fieldName = className + "." + fields[j].getName();
					

					try{
						Object obj = fields[j].get(a_object);

						if(obj.getClass().isArray()){
						// if(fields[j].getType().isArray()){
							int len = Array.getLength(obj);
							for (int k = 0 ; k < len; k ++){
								Object element = Array.get(obj,k);
								Class arrClass = element==null ? null:element.getClass();
								log(a_container, element,arrClass,fieldName, a_depth + 1, k);
							}
						}else{
							log(a_container, obj, fields[j].getType(),fieldName, a_depth + 1, -1);
						}
					}catch(Exception e){

					}
				}
			}
		}
	}

	static void log(ObjectContainer a_container, Object a_object, Class a_Class, String a_fieldName, int a_depth, int a_arrayElement){
		if(a_depth > maximumDepth){
			return;
		}
		String fieldName = (a_arrayElement > -1) ? a_fieldName + sp + sp + a_arrayElement: a_fieldName;
		if(a_object != null){
			if((a_container == null) || a_container.ext().isStored(a_object)){
				if(a_container == null || a_container.ext().isActive(a_object)){
					log(a_depth, fieldName, "");
					Class clazz = a_object.getClass();
					if(clazz == Integer.class ||
					   clazz == Long.class ||
					   clazz == Float.class ||
					   clazz == Boolean.class ||
					   clazz == Double.class ||
					   clazz == Byte.class ||
					   clazz == Character.class ||
					   clazz == Short.class ||
					   clazz == String.class ||
					   clazz == Date.class
						){
						log(a_depth + 1,a_object.getClass().getName(), a_object.toString());
					}else{
						log(a_container, a_object, a_depth);
					}
				}else{
					log(a_depth, fieldName, "DEACTIVATED " + a_object.getClass().getName());
				}
				return;
			}else{
				log(a_depth, fieldName, a_object.toString());
			}
		}else{
			log(a_depth, fieldName, "[NULL]");
		}
	}

    public static void log (String a_msg) {
		if(! silent){
			out.println(a_msg + cr);
		}
    }

    public static void log (int indent, String a_property, String a_value) {
        for (int i = 0; i < indent; i++) {
            a_property = sp + sp + a_property;
        }
        log(a_property, a_value);
    }

    public static void log (String a_property, String a_value) {
        if (a_value == null)
            a_value = "[NULL]";
        log(a_property + ": " + a_value);
    }

    public static void log (boolean a_true) {
        if (a_true)
            log("true");
        else
            log("false");
    }

    public static void log (byte[] bytes) {
        log("" + bytes);
    }

    public static void log (Integer i) {
        log(i.toString());
    }

    public static void log (int i) {
        log(new Integer(i));
    }
	
    public static void log (Long l) {
        log(l.toString());
    }

    public static void log (long l) {
        log(new Long(l));
    }

	public static void log (Double d) {
        log(d.toString());
    }

    public static void log (double d) {
        log(new Double(d));
    }

    public static void log (Exception e, Object obj, String msg) {
        String l_msg;
        if (e != null) {
            l_msg = "!!! " + e.getClass().getName();
            String l_exMsg = e.getMessage();
            if (l_exMsg != null) {
                l_msg += sp + l_exMsg;
            }
        }
        else {
            l_msg = "!!!Exception log";
        }
        if (obj != null) {
            l_msg += " in " + obj.getClass().getName();
        }
        if (msg != null) {
            l_msg += sp + msg;
        }
        log(l_msg);
    }
	
	static Class[] getClassHierarchy(Object a_object){
		Class[] classes = new Class[] {a_object.getClass()};
		return getClassHierarchy(classes);
	}

	static Class[] getClassHierarchy(Class[] a_classes){
		Class clazz = a_classes[a_classes.length -1].getSuperclass();
		if(clazz.equals(Object.class)){
			return a_classes;
		}
		Class[] classes = new Class[a_classes.length + 1];
		System.arraycopy(a_classes,0,classes,0,a_classes.length);
		classes[a_classes.length] = clazz;
		return getClassHierarchy(classes);
	}


}
