package com.db4o.lib;

public abstract class Tree
{
	protected Tree i_preceding;
	protected Tree i_subsequent;
	protected int i_size = 1;
	
	protected Tree add(Tree a_add){
		return add(this, a_add);
	}
	
	private final Tree add(Tree a_to, Tree a_new){
		int cmp = a_new.compare(a_to);
		if(cmp > 0){
			if(a_to.i_subsequent == null){
				a_to.i_subsequent = a_new;
				a_to.i_size ++;
				return a_to;
			}else{
				a_to.i_subsequent = add(a_to.i_subsequent, a_new);
				if(a_to.i_preceding == null){
					return rotateLeft(a_to);
				}else{
					return balance(a_to);
				}
			}
		}else if(cmp < 0 || a_new.duplicates()){
			if(a_to.i_preceding == null){
				a_to.i_preceding = a_new;
				a_to.i_size ++;
				return a_to;
			}else{
				a_to.i_preceding = add(a_to.i_preceding, a_new);
				if(a_to.i_subsequent == null){
					return rotateRight(a_to);
				}else{
					return balance(a_to);
				}
			}
		}else{
			i_size --;
			return a_to;
			// We increment on add and reduce here
			// if duplicates are not allowed and we
			// run into an equal node.
		}
	}
	
	private final Tree balance(Tree a_tree){
		int cmp = a_tree.i_subsequent.i_size - a_tree.i_preceding.i_size;
		if(cmp < -1){
			return rotateRight(a_tree);
		}else if(cmp > 1){
			return rotateLeft(a_tree);
		}else{
		    a_tree.i_size = a_tree.i_preceding.i_size + a_tree.i_subsequent.i_size + 1;
		    return a_tree;
		}
	}

	
	private final void calculateSize(){
		if(i_preceding == null){
			if (i_subsequent == null){
				i_size = 1;
			}else{
				i_size = i_subsequent.i_size + 1;
			}
		}else{
			if(i_subsequent == null){
				i_size = i_preceding.i_size + 1;
			}else{
				i_size = i_preceding.i_size + i_subsequent.i_size + 1;
			}
		}
	}
	
	abstract int compare(Tree a_to);
	
	abstract boolean duplicates();
	
	protected final Tree find(Tree a_tree){
		return find(this, a_tree);
	}

	private final Tree find(Tree a_in, Tree a_node){
		if(a_in == null){
			return null;
		}
		int cmp = a_in.compare(a_node);
		if (cmp < 0){
			return find(a_in.i_subsequent, a_node);
		}else{
			if (cmp > 0){
			   return find(a_in.i_preceding, a_node);
			}else{
				return a_in;
			}
		}
	}
	
	private final Tree rotateLeft(Tree a_tree){
		Tree tree = a_tree.i_subsequent;
		a_tree.i_subsequent = tree.i_preceding;
		a_tree.calculateSize();
		tree.i_preceding = a_tree;
		tree.calculateSize();
		return tree;
	}

	private final Tree rotateRight(Tree a_tree){
		Tree tree = a_tree.i_preceding;
		a_tree.i_preceding = tree.i_subsequent;
		a_tree.calculateSize();
		tree.i_subsequent = a_tree;
		tree.calculateSize();
		return tree;
	}
	
	public int size(){
		return i_size;
	}
}
