package  com.db4o.test;

import java.lang.reflect.*;
import com.db4o.lib.*;
import com.db4o.test.types.*;
import com.db4o.types.*;
import com.db4o.*;
import com.db4o.config.*;
import com.db4o.ext.*;

import  java.io.*;


public class Regression {

	/**
	 * uses a small subset of classes
	 * see method testClasses at the end of this file
	 */
    private static boolean debug = false;

	/**
	 * no comparisons, used to time performance only
	 */
    private static boolean profileOnly = false;

	/**
	 * number of regression runs
	 * use higher values for threads or debug runs
	 * use 1 for the normal run
	 */
	private static final int runs = 1;

	/**
	 * run the normal JDK1 test on all classes
	 */
	private static boolean normal = true;

	/**
	 * run JDK2 tests
	 */
    private static boolean jdk2 = true;

	/**
	 * run the thread test
	 */
	private static boolean threads = true;
	
	/**
	 * run on a MemoryFile
	 */
	private static boolean useMemoryFile = false;


    public static boolean deactivate = false;
	public static String file = "regression.yap";
	private static String errors = "";
	private static int openedThreads = 0;
	private static int returnedThreads = 0;
	private static MemoryFile memoryFile;

	private boolean closeFile;

	public synchronized static void addError(String err){
		for(int i = 0 ; i < expectedErrors.length; i++){
			if(err.equals(expectedErrors[i])){
				return;
			}
		}
		errors = errors + err + System.getProperty("line.separator");
	}

	public static void main (String[] args) {
		
		// Db4o.licensedTo("carl@db4o.com");
		
		// Db4o.configure().messageLevel(1);
		// Db4o.configure().password("Houilo7");
		// Db4o.configure().encrypt(true);
		
		new java.io.File(file).delete();
		memoryFile = new MemoryFile();
		
		new Regression().run();

		if(threads){
			new Thread(new Thread1()).start();
			new Thread(new Thread2()).start();
			new Thread(new Thread3()).start();
		}
	}
	
	public void run(){
		if (Platform.canSetAccessible() && jdk2) {
			Thread.currentThread().setName("JDK2 Regression Test");
			try{
				Regression regression2 =
					(Regression)Class.forName(
						"com.db4o.test.test2.Regression2").newInstance();
				run1(regression2.testClasses());
			}catch(Exception e){
			}
		}
		
		if( normal ) {
			Thread.currentThread().setName("Normal Regression Test");
			run1(testClasses());
		}
	}


	public void run1(RTestable[] clazzes){
		openedThreads ++;
        long time = System.currentTimeMillis();
        mainLoop(clazzes);
        time = System.currentTimeMillis() - time;
        System.out.println(Thread.currentThread().getName() +  ": " + time + " ms.");
		returnedThreads++;
		if(returnedThreads >= openedThreads){
			if (errors.length() == 0) {
				ObjectContainer con = open();
				int objectCount = con.get(null).size();
				con.close();
			    System.out.println(profileOnly ?
					"Profile run completed." :
					"Regression Test Passed. " + objectCount + " objects.");
			}
			else {
			    System.out.println("!!! Regression Test Failed. !!!");
			}
			System.out.println(Db4o.version());
			System.out.println(errors);
		}
    }

    public void mainLoop (RTestable[] clazzes) {
        int run = 0;
        for (int k = 0; k < runs; k++) {
            run++;
            System.out.println(Thread.currentThread().getName()
				+ "   Run:" + run);
            closeFile = false;
            for (int i = 0; i < 2; i++) {
                for (int j = 0; j < clazzes.length; j++) {
                    cycle(clazzes[j], run);
                }
                closeFile = true;
            }
        }

		// no duplicates
		ObjectContainer con = open();
		StoredClass[] intClasses = con.ext().storedClasses();
		String[] noDuplicates = new String[intClasses.length];
		for(int i=0; i < intClasses.length; i++){
			noDuplicates[i] = intClasses[i].getName();
		}
		for(int i = 0; i < noDuplicates.length; i++){
			for(int j = i+1; j < noDuplicates.length; j++){
				if(noDuplicates[i].equals(noDuplicates[j])){
					addError("Duplicate class definition: " + noDuplicates[i]);
				}
			}
		}
		con.close();
    }

    void cycle (RTestable clazz, int a_run) {
    	System.out.println("Testing class:" + clazz.getClass().getName());
        if (clazz.jdk2() && (!Platform.canSetAccessible())) {
            return;
        }
        Object obj = clazz.newInstance();
		ObjectContainer con = open();

		// remove all
        Object get = clazz.newInstance();
        con.deactivate(get, Integer.MAX_VALUE);
        ObjectSet set = con.get(get);
        while (set.hasNext()) {
            con.delete(set.next());
        }
        

		// STEP1 add one object
        clazz.set(obj, 1);
        con.set(obj);
        // specificTest(clazz, con, ONE);
        con = close(con);
        
        /*
        set = con.get(null);
        System.out.println(set.size());
        while(set.hasNext()){
        	Logger.log(con, set.next());
        }
        */
        
		// check 1, retrieving no members set
        compare(con, get, clazz, 1, 1);

		// check 1, retrieving all members set
        clazz.set(get, 1);
        compare(con, get, clazz, 1, 1);
        con = close(con);

		// add 4
        for (int i = 0; i < 4; i++) {
            obj = clazz.newInstance();
            clazz.set(obj, 1);
            con.set(obj);
        }
        con = close(con);

		// check 5
        compare(con, get, clazz, 1, 5);
        // specificTest(clazz, con, FIVE);
        con = close(con);

		// delete 1
        set = con.get(get);
        obj = set.next();
        con.delete(obj);
        con = close(con);

		// check 4
        compare(con, get, clazz, 1, 4);
        // specificTest(clazz, con, DELETED);
        con = close(con);

		// update 1
        set = con.get(get);
        obj = set.next();
        clazz.set(obj, 2);
        con.set(obj);
        con = close(con);

		// check 3
        compare(con, get, clazz, 1, 3);
        // specificTest(clazz, con, SAME);
        con = close(con);

		// check 1
        clazz.set(get, 2);
        compare(con, get, clazz, 2, 1);
        // specificTest(clazz, con, UPDATED);
        con = close(con);

		if(clazz.ver3()){

			// update another 1 with ver3
			set = con.get(get);
			obj = set.next();
			clazz.set(obj, 3);
			con.set(obj);
			con = close(con);

			// check 2
			clazz.set(get, 1);
			compare(con, get, clazz, 1, 3);
			// specificTest(clazz, con, SAME);
			con = close(con);

			// check 1
			clazz.set(get, 3);
			compare(con, get, clazz, 3, 1);
			// specificTest(clazz, con, UPDATED);
			con.close();
		}

		con.close();
    }

    public void compare (ObjectContainer con, Object get, RTestable clazz,
            int ver, int count) {
        ObjectSet set = con.get(get);
        if (!profileOnly) {
			set.reset();
			if (set.size() == count) {
			    while (set.hasNext()) {
			        Object res = set.next();
			        clazz.compare(con, res, ver);
			        if (deactivate) {
			            con.deactivate(res, 1);
			            con.activate(res,Integer.MAX_VALUE);
			            clazz.compare(con, res, ver);
			        }
			    }
			}
			else {
				Regression.addError(clazz.getClass().getName() + ":offcount:expected"
			            + count + ":actual:" + set.size());
			}
		}
    }

    void specificTest (Object clazz, ObjectContainer con, int step) {
        String methodName = "specific";
        Class[] parameterClasses =  {
            ObjectContainer.class, Integer.TYPE
        };
        try {
            Method method = clazz.getClass().getMethod(methodName, parameterClasses);
            if (method != null) {
                 method.invoke(clazz, new Object[] {con, new Integer(step)});
            }
        } catch (Exception e) {}
    }

    public ObjectContainer open () {
        return openContainer();
    }

	public void configure(){
        Configuration config = Db4o.configure();
		// Set ActivationDepth deep enough for Recursive classes.
		config.activationDepth(12);
        ObjectClass oc = config.objectClass("com.db4o.test.DeepUpdate");
        oc.updateDepth(2);
		oc = config.objectClass("com.db4o.test.CustomConstructor");
		oc.translate(new TCustomConstructor());
		oc = config.objectClass("com.db4o.test.Debug");
		oc.updateDepth(5);
	}

    public ObjectContainer openContainer () {
		configure();
		if(useMemoryFile){
			return ExtDb4o.openMemoryFile(memoryFile);
		}else{
			return Db4o.openFile(file);
		}
    }

    public ObjectContainer close (ObjectContainer con) {
        if (closeFile) {
            con.close();
            return  open();
        }
        return  con;
    }

    Object newInstance (Class a_class) {
        try {
            return  a_class.newInstance();
        } catch (Throwable t) {
            try {
                Constructor[] constructors = a_class.getDeclaredConstructors();
                for (int i = 0; i < constructors.length; i++) {
                    try {
                        Platform.setAccessible(constructors[i]);
                        Class[] pTypes = constructors[i].getParameterTypes();
                        Object[] parms = new Object[constructors[i].getParameterTypes().length];
                        for (int j = 0; j < parms.length; j++) {
                            for (int k = 0; k < simpleNullWrappers.length; k++) {
                                if (pTypes[j] == simpleClasses[k]) {
                                    parms[j] = simpleNullWrappers[k];
                                    break;
                                }
                            }
                        }
                        Object res = constructors[i].newInstance(parms);
                        if (res != null) {
                            return  res;
                        }
                    } catch (Exception exc) {
                        System.out.println(exc.getClass().getName());
                        System.out.println(exc.getMessage());
                    }
                }
            } catch (Exception ex) {}
            System.out.println("NewInstance failed:" + a_class.getName());
            return  null;
        }
    }

    // The following errors are expected.
    // They occur due to the fact that:
    // - Byte Objects will be instantiated even if the stored Byte object was null
    // - the "Empty" objects does not change on update
    // - the RecursiveTyped objects create more instances of themselves than would be expected
    static String[] expectedErrors =  {
        "1e3==null:com.db4o.test.types.ArrayTypedPrivate:oByte:",
		"1e0==null:com.db4o.test.types.ArrayTypedPrivate:nByte:",
		"1e3==null:com.db4o.test.types.ArrayTypedPublic:oByte:",
		"1e0==null:com.db4o.test.types.ArrayTypedPublic:nByte:",
		"com.db4o.test.types.Empty:offcount:expected3:actual:4",
		"com.db4o.test.types.Empty:offcount:expected1:actual:4",
		"f1==null:com.db4o.test.types.MasterMonster:ooo:nByte:",
		"1e3==null:com.db4o.test.types.MasterMonster:ooo:oByte:",
		"1e0==null:com.db4o.test.types.MasterMonster:ooo:nByte:",
		"com.db4o.test.types.RecursiveTypedPrivate:offcount:expected1:actual:11",
		"com.db4o.test.types.RecursiveUnTypedPrivate:offcount:expected1:actual:11",
		"com.db4o.test.types.RecursiveTypedPublic:offcount:expected1:actual:11",
		"com.db4o.test.types.RecursiveUnTypedPublic:offcount:expected1:actual:11",
		"f1==null:com.db4o.test.types.TypedPrivate:nByte:",
		"f1==null:com.db4o.test.types.TypedPublic:nByte:"
	};

    static Object[] simpleNullWrappers =  {
        new Integer(0), new Long(0), new Character((char)0), new Double((double)0),
                new Float((float)0), new Boolean(false), new Short((short)0),
                new Byte((byte)0)
    };

    static Class[] simpleClasses =  {
        Integer.TYPE, Long.TYPE, Character.TYPE, Double.TYPE, Float.TYPE, Boolean.TYPE,
                Short.TYPE, Byte.TYPE
    };


    public static final int ONE = 1;
    public static final int FIVE = 5;
    public static final int DELETED = 4;
    public static final int SAME = 3;
    public static final int UPDATED = 0;

    public static RTestable[] allClasses () {
        return  new RTestable[] {
            new ArrayInObjectPrivate(),
            new ArrayInObjectPublic(),
            new ArrayMixedInObjectPrivate(),
            new ArrayMixedInObjectPublic(),
            new ArrayMixedTypedPrivate(),
            new ArrayMixedTypedPublic(),
            new ArrayNDimensionalPrivate(),
            new ArrayNDimensionalPublic(),
            new ArrayTypedPrivate(),
            new ArrayTypedPublic(),
            new ArrayUntypedPrivate(),
            new ArrayUntypedPublic(),
            new BiParentTypedPrivate(),
            new BiParentTypedPublic(),
            new BiParentUnTypedPrivate(),
            new BiParentUnTypedPublic(),
            new DeepUpdate(),
			new Empty(),
            new InterfacePrivate(),
            new InterfacePublic(),
            new ObjectSimplePrivate(),
            new ObjectSimplePublic(),
            new ParameterConstructor(0),
            PrivateConstructor.construct(),
            new RecursiveTypedPrivate(),
            new RecursiveTypedPublic(),
            new RecursiveUnTypedPrivate(),
            new RecursiveUnTypedPublic(),
            new RHashtable(),
            new RProperties(),
            new RStack(),
            new RVector(),
			new SelfReference(),
            new TypedPrivate(),
            new TypedPublic(),
            new UntypedPrivate(),
            new UntypedPublic()
			// new MasterMonster()
        };
    }

    public RTestable[] testClasses () {
		if(!debug){
			return allClasses();
		}
		return  new RTestable[] {
			
			new CallBacksChild()
			/*
            new InterfacePrivate(),
            new InterfacePublic(),
            new ObjectSimplePrivate(),
            */
            // new ExtendsDate()
            
            /*,
            new ArrayInObjectPublic(),
            new ArrayMixedInObjectPrivate(),
            new ArrayMixedInObjectPublic(),
            new ArrayMixedTypedPrivate(),
            new ArrayMixedTypedPublic(),
            new ArrayNDimensionalPrivate(),
            new ArrayNDimensionalPublic(),
            new ArrayTypedPrivate(),
            new ArrayTypedPublic(),
            */
        };
    }
}
