package com.db4o.tools;

import com.db4o.*;
import com.db4o.ext.*;
import com.db4o.lib.IntTree;
import java.util.Vector;
import java.util.Enumeration;

/**
 * prints statistics about a database file to System.out.
 * <br><br>Pass the database file path as an argument.
 * <br><br>This class is not part of db4o.jar. It is delivered as sourcecode
 * in the path ../com/db4o/tools/<br><br>
 */
public class Statistics
{
	
	/**
	 * the main method that runs the statistics.
	 * @param String[] a String array of length 1, with the name of the database
	 * file as element 0.
	 */
	public static void main (String[] args){
		if(args == null || args.length != 1){
			System.out.println("Usage: java com.db4o.tools.Statistics <database filename>");
		}else{
			new Statistics().run(args[0]);
		}
	}

	private void run(String filename){
		if(new java.io.File(filename).exists()){
			ObjectContainer con = null;
			try{
				con = Db4o.openFile(filename);
				System.out.println("db4o");
				System.out.println("STATISTICS FOR FILE '" + filename + "'");
				printStats(con);
				con.close();
			}catch(Exception e){
				System.out.println("Statistics failed for file: '" + filename + "'");
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}else{
			System.out.println("File not found: '" + filename + "'");
		}
	}

	private void printStats(ObjectContainer con){
		
		Vector unavailable = new Vector();
		Vector noConstructor = new Vector();
		IntTree ids = new IntTree(0);  // one element too many, substract one in the end

		StoredClass[] internalClasses = con.ext().storedClasses();
		for(int i=0; i < internalClasses.length; i++){
			try{
				Class clazz = Class.forName(internalClasses[i].getName());
				try{
					clazz.newInstance();
				}catch(Throwable th){
					noConstructor.addElement(internalClasses[i].getName());
				}
			}
			catch(Throwable t){
				unavailable.addElement(internalClasses[i].getName());
			}
		}
		System.out.println("Stored classes: " + internalClasses.length);
		if(unavailable.size() > 0){
			System.out.println("Unavailable classes: " + unavailable.size());
		}
		if(noConstructor.size() > 0){
			System.out.println("Classes without public constructors: " + noConstructor.size());
		}
		if(unavailable.size() > 0){
			System.out.println("UNAVAILABLE");
			Enumeration enum = unavailable.elements();
			while(enum.hasMoreElements()){
				System.out.println(enum.nextElement());
			}
		}
		if(noConstructor.size() > 0){
			System.out.println("NO PUBLIC CONSTRUCTOR");
			Enumeration enum = noConstructor.elements();
			while(enum.hasMoreElements()){
				System.out.println(enum.nextElement());
			}
		}

		System.out.println("CLASSES");
		System.out.println("Number of objects per class:");

		for(int i = 0 ; i < internalClasses.length; i++){
			long[] newIDs = internalClasses[i].getIDs();
			for(int j = 0; j < newIDs.length; j++){
				if(ids.find((int)newIDs[j]) == null){
					ids = ids.add((int)newIDs[j]);
				}
			}
			System.out.println(internalClasses[i].getName() + ": " + newIDs.length);
		}

		System.out.println("SUMMARY");
		System.out.println("Total number of objects: " + (ids.size() - 1));
	}
	

}

